local BG = LibStub("AceAddon-3.0"):GetAddon("Battleground")
local module = BG:NewModule("AlteracValley", "AceEvent-3.0", "AceTimer-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("Battleground")

local landmarks = BG.landmarks

local pois = {
	right = {
		L"Iceblood Tower",
		L"Iceblood Graveyard",
		L"Tower Point",
		L"Frostwolf Graveyard",
		L"East Frostwolf Tower",
		L"West Frostwolf Tower",
		L"Frostwolf Relief Hut",
		L"Coldtooth Mine",
	},
	left = {
		L"Stonehearth Bunker",
		L"Stonehearth Graveyard",
		L"Icewing Bunker",
		L"Stormpike Graveyard",
		L"Dun Baldar North Bunker",
		L"Dun Baldar South Bunker",
		L"Stormpike Aid Station",
		L"Irondeep Mine",
	},
	top = {
		L"Battleground Start",
	},
	bottom = {
		L"Snowfall Graveyard",
	}
}
local chatFilters = {
	[L"Captain Balinda Stonehearth"] = {
		[L"Begone, uncouth scum!  The Alliance shall prevail in Alterac Valley!"] = L"The %s has engaged %s.",
		[L"Filthy Frostwolf cowards! If you want a fight, you'll have to come to me!"] = L"The %s has reset %s.",
	},
	[L"Captain Galvangar"] = {
		[L"Die!  Your kind has no place in Alterac Valley!"] = L"The %s has engaged %s.",
		[L"I'll never fall for that, fool! If you want a battle it will be on my terms and in my lair!"] = L"The %s has reset %s.",
	},
	[L"Vanndar Stormpike"] = {
		[L"Soldiers of Stormpike, your General is under attack! I require aid! Come! Come! Slay these mangy Frostwolf dogs."] = L"The %s has engaged %s.",
		[L"You'll never get me out of me bunker, heathens!"] = L"The %s has reset %s.",
	},
	[L"Drek'Thar"] = {
		[L"Stormpike filth! In my keep?! Slay them all!"] = L"The %s has engaged %s.",
		[L"You seek to draw the General of the Frostwolf legion out from his fortress? PREPOSTEROUS!"] = L"The %s has reset %s.",
	},
}
local faction = {
	[L"Captain Balinda Stonehearth"] = FACTION_HORDE,
	[L"Captain Galvangar"] = FACTION_ALLIANCE,
	[L"Vanndar Stormpike"] = FACTION_HORDE,
	[L"Drek'Thar"] = FACTION_ALLIANCE
}

-- Reinforcments
local rAlliance = 0
local rHorde = 0

-- DBM sync drek
local dbmRevision = 0
local dbmSender = nil

local function whisperDBM()
	if not frame or not dbmSender then return end
	for name, poi in pairs(frame.pois) do
		if poi.status and not poi.expire and (poi.status == "horde_assaulted" or poi.status == "alliance_assaulted") then
			SendAddonMessage("DBMv4-RequestTimers", "", "WHISPER", dbmSender)
			break
		end
	end
end

local function chat(chatFrame, msg, faction)
	local color
	if faction == FACTION_ALLIANCE then
		color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"]
	elseif faction == FACTION_HORDE then
		color = ChatTypeInfo["BG_SYSTEM_HORDE"]
	end
	chatFrame:AddMessage(msg, color.r, color.g, color.b)
end

local function filterFunc(chatFrame, event, msg, author, ...)
	if author == L["Herald"] then
		if msg:match(FACTION_ALLIANCE) then
			chat(chatFrame, msg, FACTION_ALLLIANCE)
		elseif msg:match(FACTION_HORDE) then
			chat(chatFrame, msg, FACTION_HORDE)
		else return end
		return true -- filter
	end
	if chatFilters[author] and chatFilters[author][msg] then
		chat(chatFrame, chatFilters[author][msg]:format(faction[author], author), faction[author])
		return true
	end
	return
end

local frame

local function OnUpdate(self, elapsed)
	if not self.expire then return end
	if self.expire >= time() then
		local tleft = self.expire - time()
		local m = floor(tleft/60)
		local s = tleft % 60
		self.time:SetFormattedText("%0d:%02d", m, s)
	elseif self.switchname then
		self.text:SetText(self.switchname)
		self.switchname = nil
		self.expire = time()
		self:SetScript("OnUpdate", OnUpdateTime)
	end
end

local function OnUpdateTime(self, elapsed)
	if self.expire then
		local telapsed = time() - self.expire
		local m = floor(telapsed/60)
		local s = telapsed % 60
		self.time:SetFormattedText("%0d:%02d", m, s)
	end
end


local function getButton(orientation)
	local button = CreateFrame("Button", nil, frame)
	button:Show()
	local text = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
	local time = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
	local icon = button:CreateTexture(nil, "BACKGROUND")
	button:SetWidth(16)
	button:SetHeight(16)
	text:SetWidth(200)
	text:SetHeight(16)
	time:SetWidth(200)
	time:SetHeight(16)
	button.text = text
	button.time = time
	button.icon = icon
	icon:SetPoint("CENTER", button, "CENTER")
	icon:SetWidth(16)
	icon:SetHeight(16)
	icon:SetTexture("Interface\\Minimap\\POIIcons")
	button:SetHeight(16)
	if orientation == "right" then
		time:SetPoint("RIGHT", button, "LEFT")
		text:SetPoint("LEFT", button, "RIGHT")
		time:SetJustifyH("RIGHT")
		text:SetJustifyH("LEFT")
	else
		text:SetPoint("RIGHT", button, "LEFT")
		time:SetPoint("LEFT", button, "RIGHT")
		text:SetJustifyH("RIGHT")
		time:SetJustifyH("LEFT")
	end
	return button
end

local function createGUI()
	if frame then return end
	frame = CreateFrame("Frame", nil, UIParent)
	frame:Hide()
	frame.pois = {}
	frame:SetPoint("TOP", WorldStateAlwaysUpFrame, "TOP", 30, -8) -- -66)
	frame:SetWidth(260)
	frame:SetHeight(30)
	local last
	for k, v in pairs( pois.left ) do
		frame.pois[v] = getButton("left")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("RIGHT", frame, "LEFT")
		else
			frame.pois[v]:SetPoint("TOP", last, "BOTTOM")
			last = frame.pois[v]
		end
	end
	for k, v in pairs( pois.right ) do
		frame.pois[v] = getButton("right")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("RIGHT", frame, "RIGHT")
		else
			frame.pois[v]:SetPoint("TOP", last, "BOTTOM")
			last = frame.pois[v]
		end
	end
	for k, v in pairs(pois.top) do
		frame.pois[v] = getButton("right")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("BOTTOM", frame, "TOP", -50, 0)
		else
			frame.pois[v]:SetPoint("BOTTOM", last, "TOP")
			last = frame.pois[v]
		end
	end
	for k, v in pairs(pois.bottom) do
		frame.pois[v] = getButton("right")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("TOP", frame, "TOP", -50, -56)
		else
			frame.pois[v]:SetPoint("TOP", last, "BOTTOM")
			last = frame.pois[v]
		end
	end
end

local function showGUI()
	if not frame then createGUI() end
	for k, v in pairs(frame.pois) do
		v.status = nil
	end
	frame.pois[L"Battleground Start"].icon:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
	frame.pois[L"Battleground Start"].icon:SetTexCoord(0.57, 0.90, 0.08, 0.41)
	frame:Show()
end

local function hideGUI()
	frame:Hide()
end

function module:OnEnable()
	rAlliance = 0
	rHorde = 0
	dbmRevision = 0
	dbmSender = nil
	ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_YELL", filterFunc)
	showGUI()
	self:RegisterMessage("Battleground_POIUpdate")
	self:RegisterMessage("Battleground_StartTimer")
	self:RegisterMessage("Battleground_Started")
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterEvent("UPDATE_WORLD_STATES")
	-- request sync from as many as we can
	SendAddonMessage("cap", "r", "BATTLEGROUND") -- Capping request for timers
	SendAddonMessage("DBMv4-Ver", "Hi!") -- DBM Greeting
	self:ScheduleTimer( whisperDBM, 5)
end

function module:OnDisable()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_MONSTER_YELL", filterFunc)
	hideGUI()
end

function module:Battleground_Started(message, t)
	local poi = frame.pois[L"Battleground Start"]
	poi.expire = time() - t
	poi.text:SetText(L"Elapsed")
	poi:SetScript("OnUpdate", OnUpdateTime)
end

function module:Battleground_StartTimer(message, t)
	local poi = frame.pois[L"Battleground Start"]
	poi.text:SetText(L"Battleground Start")
	poi.switchname = L"Elapsed"
	poi.expire = time() + t
	poi:SetScript("OnUpdate", OnUpdate)
end

function module:Battleground_POIUpdate(message, name, ti)
	if not frame or not frame.pois[name] then return end
	local poi = frame.pois[name]
	poi.icon:SetTexCoord( landmarks[ti][3], landmarks[ti][4], landmarks[ti][5], landmarks[ti][6])
	local status = landmarks[ti][2]
	poi:SetAlpha(1)
	poi.text:SetTextColor(.6,.6,.6)
	if status == "neutral" then
		if landmarks[ti][1] == "tower" then poi:SetAlpha(.6) end
	elseif status == "horde" then
		local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
		poi.text:SetTextColor(color.r, color.g, color.b)
	elseif status == "alliance" then
		local color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"]
		poi.text:SetTextColor(color.r, color.g, color.b)
	end
	if ( status == "alliance_assaulted" or status == "horde_assaulted" ) and poi.status then
		poi.expire = time() + 245
		local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
		if status == "alliance_assaulted" then color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"] end
		poi.time:SetTextColor(color.r, color.g, color.b)
		poi:SetScript("OnUpdate", OnUpdate)
	else
		poi.status = status
		poi.expire = nil
		poi:SetScript("OnUpdate", nil)
		poi.time:SetText(nil)
	end
end

function module:CHAT_MSG_ADDON(event, prefix, message, chan, sender)
	if prefix == "cap" and message ~= "r" then
		local name, duration, elapsed, color = strmatch(message, "^(.+)@(%d+)@(%d+)@(%a+)$")
		local poi = frame.pois[name]
		if poi and not poi.expire then
			poi.expire = time() + (duration - elapsed)
			local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
			if poi.status == "alliance_assaulted" then color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"] end
			poi.time:SetTextColor(color.r, color.g, color.b)
			poi:SetScript("OnUpdate", OnUpdate)
		end
	elseif prefix == "DBMv4-Ver" and message ~= "Hi!" then
		local revision, version, displayVersion, locale = strsplit("\t", msg)
		revision = tonumber(revision or "")
		if dbmRevision < revision then
			dbmRevision = revision
			dbmSender = sender
		end
	elseif prefix == "DBMv4-TimerInfo" then
		print(strsplit("\t", message))
		--[[ 
		local mod, timeLeft, totalTime, id = strsplit("\t", msg)
		mod = DBM:GetModByName(mod or "")
		timeLeft = tonumber(timeLeft or 0)
		totalTime = tonumber(totalTime or 0)
		if mod and timeLeft and timeLeft > 0 and totalTime and totalTime > 0 and id then
			DBM:ReceiveTimerInfo(sender, mod, timeLeft, totalTime, id, select(5, strsplit("\t", msg)))
		end
		--]]
	end
end

function module:UPDATE_WORLD_STATES()
	local alliance = select(3, GetWorldStateUIInfo(1))
	local horde = select(3, GetWorldStateUIInfo(2))
	if not alliance or not horde then return end
	
	local nr = tonumber(alliance:match("([0-9]+)")) or 0
	if (rAlliance - nr) == 100 then chat(DEFAULT_CHAT_FRAME, L["The %s has defeated %s."]:format(FACTION_HORDE, L["Captain Balinda Stonehearth"]), FACTION_HORDE) end
	rAlliance = nr
	nr = tonumber(horde:match("([0-9]+)")) or 0
	if (rHorde - nr) == 100 then chat(DEFAULT_CHAT_FRAME, L["The %s has defeated %s."]:format(FACTION_ALLIANCE, L["Captain Galvangar"]), FACTION_ALLIANCE) end
	rHorde = nr
end
